@@ -50,8 +50,6 @@ namespace Banshee.Gui
 {
     public class SourceActions : BansheeActionGroup
     {
-        private InterfaceActionService action_service;
-
         private IHasSourceView source_view;
         public IHasSourceView SourceView {
             get { return source_view; }
@@ -66,10 +64,8 @@ namespace Banshee.Gui
             get { return (SourceView.HighlightedSource as PrimarySource) ?? base.ActivePrimarySource; }
         }
 
-        public SourceActions (InterfaceActionService actionService) : base ("Source")
+        public SourceActions (InterfaceActionService actionService) : base (actionService, "Source")
         {
-            action_service = actionService;
-
             Add (new ActionEntry [] {
                 new ActionEntry ("NewPlaylistAction", null,
                     Catalog.GetString ("_New Playlist"), "<control>N",
@@ -91,22 +87,21 @@ namespace Banshee.Gui
                     Catalog.GetString ("Import source to library"), OnImportSource),
 
                 new ActionEntry ("RenameSourceAction", null,
-                    "Rename", "F2", "Rename", OnRenameSource),
+                    Catalog.GetString ("Rename"), "F2", Catalog.GetString ("Rename"), OnRenameSource),
 
                 new ActionEntry ("ExportPlaylistAction", null,
                     Catalog.GetString ("Export Playlist..."), null,
                     Catalog.GetString ("Export a playlist"), OnExportPlaylist),
 
                 new ActionEntry ("UnmapSourceAction", null,
-                    "Unmap", "<shift>Delete", null, OnUnmapSource),
+                    Catalog.GetString ("Unmap"), "<shift>Delete", null, OnUnmapSource),
                     
                 new ActionEntry ("SourcePropertiesAction", null,
-                    "Source Properties", null, null, OnSourceProperties),
+                    Catalog.GetString ("Source Properties"), null, null, OnSourceProperties),
             });
 
             this["NewPlaylistAction"].IconName = Stock.New;
             this["UnmapSourceAction"].IconName = Stock.Delete;
-            this["RenameSourceAction"].IconName = "gtk-edit";
             this["SourcePropertiesAction"].IconName = Stock.Properties;
 
             AddImportant (
@@ -141,7 +136,7 @@ namespace Banshee.Gui
             //ServiceManager.SourceManager.SourceAdded += OnPlayerEngineStateChanged;
             //ServiceManager.SourceManager.SourceRemoved += OnPlayerEngineStateChanged;
             ServiceManager.SourceManager.ActiveSourceChanged += HandleActiveSourceChanged;
-            action_service.GlobalActions["EditMenuAction"].Activated += HandleEditMenuActivated;
+            Actions.GlobalActions["EditMenuAction"].Activated += HandleEditMenuActivated;
         }
             
 #region State Event Handlers
@@ -149,12 +144,25 @@ namespace Banshee.Gui
         private void HandleActiveSourceChanged (SourceEventArgs args)
         {
             UpdateActions ();
+            
+            if (last_source != null) {
+                last_source.Updated -= HandleActiveSourceUpdated;
+            }
+            
+            if (ActiveSource != null) {
+                ActiveSource.Updated += HandleActiveSourceUpdated;
+            }
         }
 
         private void HandleEditMenuActivated (object sender, EventArgs args)
         {
             UpdateActions ();
         }
+        
+        private void HandleActiveSourceUpdated (object o, EventArgs args)
+        {
+            UpdateActions (true);
+        }
 
 #endregion
 
@@ -165,7 +173,7 @@ namespace Banshee.Gui
             PlaylistSource playlist = new PlaylistSource ("New Playlist", ActivePrimarySource.DbId);
             playlist.Save ();
             playlist.PrimarySource.AddChildSource (playlist);
-            playlist.NotifyUpdated ();
+            playlist.NotifyUser ();
             //SourceView.BeginRenameSource (playlist);
         }
 
@@ -212,8 +220,8 @@ namespace Banshee.Gui
         {
             UpdateActions ();
 
-            string path = ActionSource.Properties.GetString ("GtkActionPath") ?? "/SourceContextMenu";
-            Gtk.Menu menu = action_service.UIManager.GetWidget (path) as Menu;
+            string path = ActionSource.Properties.Get<string> ("GtkActionPath") ?? "/SourceContextMenu";
+            Gtk.Menu menu = Actions.UIManager.GetWidget (path) as Menu;
             if (menu == null || menu.Children.Length == 0) {
                 SourceView.ResetHighlight ();
                 UpdateActions ();
@@ -238,7 +246,7 @@ namespace Banshee.Gui
                 UpdateActions ();
             };
         }
-            
+
         private void OnImportSource (object o, EventArgs args)
         {
             (ActionSource as IImportSource).Import ();
@@ -283,7 +291,7 @@ namespace Banshee.Gui
                 playlist.Save (Banshee.IO.File.OpenWrite (new SafeUri (uri), true), source);
             } catch (Exception e) {
                 Console.WriteLine (e);
-                Log.Error (Catalog.GetString ("Could not export playlist"), e.Message);
+                Log.Error (Catalog.GetString ("Could not export playlist"), e.Message, true);
             }
         }
 
@@ -305,6 +313,11 @@ namespace Banshee.Gui
 
         private void OnSourceProperties (object o, EventArgs args)
         {
+            if (ActionSource.Properties.Contains ("SourceProperties.GuiHandler")) {
+                ActionSource.Properties.Get<Source.OpenPropertiesDelegate> ("SourceProperties.GuiHandler") ();
+                return;
+            }
+
             SmartPlaylistSource source = ActionSource as SmartPlaylistSource;
             if (source != null) {
                 Editor ed = new Editor (source);
@@ -318,21 +331,41 @@ namespace Banshee.Gui
 
         private Source last_source = null;
         private void UpdateActions ()
+        {
+            UpdateActions (false);
+        }
+        
+        private void UpdateActions (bool force)
         {
             Source source = ActionSource;
 
-            if (source != last_source && source != null) {
-                IUnmapableSource unmapable = source as IUnmapableSource;
-                IImportSource import_source = source as IImportSource;
-                SmartPlaylistSource smart_playlist = source as SmartPlaylistSource;
+            if ((force || source != last_source) && source != null) {
+                IUnmapableSource unmapable = (source as IUnmapableSource);
+                IImportSource import_source = (source as IImportSource);
+                SmartPlaylistSource smart_playlist = (source as SmartPlaylistSource);
+                PrimarySource primary_source = (source as PrimarySource) ?? (source.Parent as PrimarySource);
+
                 UpdateAction ("UnmapSourceAction", unmapable != null, unmapable != null && unmapable.CanUnmap, source);
                 UpdateAction ("RenameSourceAction", source.CanRename, true, null);
                 UpdateAction ("ImportSourceAction", import_source != null, import_source != null && import_source.CanImport, source);
                 UpdateAction ("ExportPlaylistAction", source is AbstractPlaylistSource, true, source);
                 UpdateAction ("SourcePropertiesAction", source.HasProperties, true, source);
                 UpdateAction ("RefreshSmartPlaylistAction", smart_playlist != null && smart_playlist.CanRefresh, true, source);
+
+                UpdateAction ("NewPlaylistAction", primary_source != null && primary_source.SupportsPlaylists, true, source);
+                UpdateAction ("NewSmartPlaylistAction", primary_source != null && primary_source.SupportsPlaylists, true, source);
                 /*UpdateAction ("NewSmartPlaylistFromSearchAction", (source is LibrarySource || source.Parent is LibrarySource),
                         !String.IsNullOrEmpty (source.FilterQuery), source);*/
+                    
+                ActionGroup browser_actions = Actions.FindActionGroup ("BrowserView");
+                if (browser_actions != null) {
+                    IFilterableSource filterable_source = source as IFilterableSource;
+                    bool has_browser = filterable_source != null && filterable_source.AvailableFilters.Count > 0;
+                    UpdateAction (browser_actions["BrowserTopAction"], has_browser);
+                    UpdateAction (browser_actions["BrowserLeftAction"], has_browser);
+                    UpdateAction (browser_actions["BrowserVisibleAction"], has_browser);
+                }
+
                 last_source = source;
             }
             
