@@ -99,7 +99,7 @@ namespace Banshee.Podcasting
                     Hyena.Log.Information ("Migrating Podcast Feeds and Items");
                     ServiceManager.DbConnection.Execute(@"
                         INSERT INTO PodcastSyndications (FeedID, Title, Url, Link,
-                            Description, ImageUrl, LastBuildDate, SyncSetting, IsSubscribed)
+                            Description, ImageUrl, LastBuildDate, AutoDownload, IsSubscribed)
                             SELECT 
                                 PodcastFeedID,
                                 Title,
@@ -172,6 +172,21 @@ namespace Banshee.Podcasting
                 DatabaseConfigurationClient.Client.Set<int> ("Podcast", "Version", 1);
             }
 
+            if (DatabaseConfigurationClient.Client.Get<int> ("Podcast", "Version", 0) < 3) {
+                // We were using the Link as the fallback if the actual Guid was missing, but that was a poor choice
+                // since it is not always unique.  We now use the title and pubdate combined.
+                ServiceManager.DbConnection.Execute ("UPDATE PodcastItems SET Guid = NULL");
+                foreach (FeedItem item in FeedItem.Provider.FetchAll ()) {
+                    item.Guid = null;
+                    if (item.Feed == null || FeedItem.Exists (item.Feed.DbId, item.Guid)) {
+                        item.Delete (false);
+                    } else {
+                        item.Save ();
+                    }
+                }
+
+                DatabaseConfigurationClient.Client.Set<int> ("Podcast", "Version", 3);
+            }
         }
         
         public void Initialize ()
@@ -235,12 +250,13 @@ namespace Banshee.Podcasting
             }
             
             // Handle podcast URIs
-            if (uri.StartsWith ("feed://")) {
-                string url = String.Format ("http://{0}", uri.Substring (7));
-                Log.DebugFormat ("Subscribing to podcast at {0}", url);
+            if (uri.Contains ("xml") || uri.Contains ("rss") || uri.Contains ("feed") || uri.StartsWith ("itpc")) {
+                if (uri.StartsWith ("feed://") || uri.StartsWith ("itpc://")) {
+                    uri = String.Format ("http://{0}", uri.Substring (7));
+                }
 
                 // TODO replace autodownload w/ actual default preference
-                FeedsManager.Instance.FeedManager.CreateFeed (url, FeedAutoDownload.None);
+                FeedsManager.Instance.FeedManager.CreateFeed (uri, FeedAutoDownload.None);
                 source.NotifyUser ();
             }
         }
